use std::{vec,fs::File,sync::Arc};

use opentelemetry::KeyValue;
use opentelemetry_otlp::WithExportConfig;
use opentelemetry_sdk::{runtime, trace, Resource};
use time::format_description;
use tracing::level_filters::LevelFilter;
use tracing::subscriber;
use tracing_bunyan_formatter::{BunyanFormattingLayer, JsonStorageLayer};
use tracing_opentelemetry::OpenTelemetryLayer;
use tracing_subscriber::{filter, fmt, Layer, Registry};
use super::tracing_kind::{Tracing, TracingKind};
use tracing_subscriber::fmt::time::UtcTime;
use tracing_subscriber::prelude::__tracing_subscriber_SubscriberExt;

pub fn init_tracing(tracing_config: Vec<Tracing>,name: String){
    let mut layers = vec![];
    for config in tracing_config {
        match config.kind {
            TracingKind::File => {
                let file = File::options()
                    .create(true)
                    .truncate(false)
                    .append(true)
                    .open("trace.log")
                    .expect("Failed to create trace.log");
                let formating_layer = BunyanFormattingLayer::new(name.clone(), Arc::new(file)).boxed();
                layers.push(JsonStorageLayer.boxed());
                layers.push(formating_layer);
            }
            TracingKind::Console => {
                let time_format = format_description::parse("[hour]:[minute]:[second]")
                    .expect("format string should be valid!");
                let timer = UtcTime::new(time_format);

                let terminal_out = fmt::layer()
                    .with_thread_names(true)
                    .with_timer(timer)
                    .with_target(false)
                    .with_filter(LevelFilter::from(config.level)).boxed();
                layers.push(terminal_out);
            }
            TracingKind::Otel => {
                let endpoint = match config.additional.get("endpoint"){
                    Some(endpoint) => endpoint.to_string(),
                    None => "http://localhost:4317".to_string()
                };
                let pod_name = std::env::var("POD_NAME").unwrap_or_else(|_| "not_a_pod".to_string());
                let telemetry = opentelemetry_otlp::new_pipeline()
                    .tracing()
                    .with_exporter(opentelemetry_otlp::new_exporter().tonic().with_endpoint(endpoint))
                    .with_trace_config(trace::config().with_resource(Resource::new(vec![
                        KeyValue::new("service.name", name.clone()),
                        KeyValue::new("service.pod", pod_name.clone()),
                    ])))
                    .install_batch(runtime::Tokio)
                    .expect("Failed to install opentelemetry");
                let tele_layer = OpenTelemetryLayer::new(telemetry).with_filter(filter::LevelFilter::from(config.level));
                layers.push(tele_layer.boxed());
                println!("Otel Tracing not implemented yet");
            }
        }
    }
    subscriber::set_global_default(Registry::default().with(layers))
            .expect("setting default subscriber failed");
}


pub fn stop_tracing(tracing_config: Vec<Tracing>,_name: String){
    if tracing_config.iter().any(|x| x.kind == TracingKind::Otel){
        opentelemetry::global::shutdown_tracer_provider();
    }
}